C***********************************************************************
C
C This block of routines permits use of the incoherent scattering 
C function in determining the compton scattering angle, using the logic 
C and coding in  PENELOPE.  
C
C last modification:
C 1999-11-11
C***********************************************************************
C
C  section INSCATI
C
      subroutine INSCATI(mname,len,m)
C                                                                       
C  Primary purpose:  To read the inscat data from the penelope material
C                    files.
C
C  Input required:
C    Through arg list:
C    MNAME     - the name of the material
C    LEN       - number of characters in the name
C    M         - which material number we're on
C
C  Output:
C    Through labeled common XHIN:
C      NOSC(M)   - number of oscillators
C      ZT(10)    - number of electrons
C      FZ(M,IO)  - incoherent scattering function at each osc
C      BEN(M,IO) - energy of the oscillator
C      FJ0(M,IO) - one electron compton profile
C
C  Declarations for Globals:
C
      character*40 mname
      integer len,m
C
      integer nosc
      double precision fz,ben,fj0,zt
      common/XHIN/FZ(10,30),BEN(10,30),NOSC(10),FJ0(10,30),ZT(10)
C
C  declarations for locals and dummys...
C
      double precision sl
      PARAMETER(SL=137.0359895D0)
C
      character*80 buffer
      integer i,j,idummy
      double precision dummy,f(30), ui(30), ft, save
C    
C  open the file...
C
      buffer='_pendat/'//mname(1:len)
      open(7,file=buffer)
C
C  read a lot of dummy matter...
C
C title...
      read(7,'(a100)') buffer
C density...
      read(7,'(a100)') buffer
C composition...
      READ(7,1040) j
 1040 FORMAT(41X,I2)
C elemental data...
      ZT(M)=0.0D0
      DO 3 I=1,J
        READ(7,1050) idummy,dummy
        ZT(M)=ZT(M)+idummy*dummy
 1050 FORMAT(20X,I2,18X,E13.6)
    3 CONTINUE
C mean excitation energy...
      read(7,'(a100)') buffer
C oscillator data...
      READ(7,1070) NOSC(M)
 1070 FORMAT(28X,I2)
      DO 4 I=1,NOSC(M)
        READ(7,1080) F(I),UI(I),FJ0(M,I),idummy
 1080 FORMAT(6X,E13.6,1X,E13.6,1X,E10.3,1X,I2)
    4 CONTINUE
C  ****  OSCILLATORS ARE SORTED BY INCREASING ENERGIES.
      FT=0.0D0
      IF(NOSC(M).GT.1) THEN
        DO 6 I=1,NOSC(M)-1
        DO 5 J=I+1,NOSC(M)
        IF(UI(I).LT.UI(J)) GO TO 5
        SAVE=UI(I)
        UI(I)=UI(J)
        UI(J)=SAVE
        SAVE=F(I)
        F(I)=F(J)
        F(J)=SAVE
        SAVE=FJ0(M,I)
        FJ0(M,I)=FJ0(M,J)
        FJ0(M,J)=SAVE
    5   CONTINUE
        FT=FT+F(I)
    6   CONTINUE
      ELSE
        UI(1)=DABS(UI(1))
      ENDIF
      FT=FT+F(NOSC(M))
C  ****  QUANTITIES STORED IN COMMON BLOCKS.
      DO 10 I=1,NOSC(M)
        FZ(M,I)=F(I)/FT
        BEN(M,I)=DMAX1(UI(I),0.0D0)
        FJ0(M,I)=FJ0(M,I)*SL
   10 CONTINUE
C
      close(7)
      return
      end
C
C***********************************************************************
C                                                                       
C  Section SINCOH
C
      subroutine SINCOH(m,energy,efrac,costhe)
C                                                                       
C  Primary purpose:  To get a compton energy and angle using the
C                    incoherent scattering functions.
C
C  Input required:
C    through argument list:
C      ENERGY    - duh.
C      M         - current material number
C
C    Through labeled common XHIN:
C      NOSC(M)   - number of oscillators
C      ZT(10)    - number of electrons
C      FZ(M,IO)  - incoherent scattering function at each osc
C      BEN(M,IO) - energy of the oscillator
C      FJ0(M,IO) - one electron compton profile
C
C  Output:
C    through argument list:
C      EFRAC     - fraction of initial energy kept by photon
C      COSTHE    - cosine of scattering angle
C
C  Declarations for Global variables:
C
      integer m
      double precision energy,efrac,costhe
C
      integer nosc
      double precision fz,ben,fj0,zt
      common/XHIN/FZ(10,30),BEN(10,30),NOSC(10),FJ0(10,30),ZT(10)
C
C  Declarations for local variables:
C
      double precision rev,d1,d2,d12
      PARAMETER (REV=5.1099906D5)
      PARAMETER (D1=0.707107D0, D2=1.414214D0, D12=D1*D1)
C
      integer i,ish,nsm
      double precision rng
      double precision ek,ek1,ek2,ek3,tau,tau0,a1,a2,s0,aux,pzomc
      double precision s,tst,argtes,rni,rn(30),pac(30),cdt1
      double precision a,b1,b2,ep,af,xqc,fpzmax,fpz,t
C
      EK=energy/REV
      EK2=EK+EK+1.0D0
      EK3=EK*EK
      EK1=EK3-EK2-1.0D0
      TAU0=1.0D0/EK2
      A1=DLOG(EK2)
      A2=A1+2.0D0*EK*(1.0D0+EK)/(EK2*EK2)
      NSM=NOSC(M)
      IF(energy.GT.5.0D6) GO TO 7
C
C  ****  INCOHERENT SCATTERING FUNCTION FOR THETA=PI.
C
      S0=0.0D0
      DO 1 I=1,NSM
      IF(BEN(M,I).LT.energy) THEN
        AUX=energy*(energy-BEN(M,I))*2.0D0
        PZOMC=FJ0(M,I)*(AUX-REV*BEN(M,I))
     1       /(REV*DSQRT(AUX+AUX+BEN(M,I)**2))
C
        IF(PZOMC.GT.0.0D0) THEN
          ARGTES = D12-(D1+D2*PZOMC)**2
          IF(ARGTES.LT.-2.0D1) THEN
            RNI = 1.0D0
          ELSE
            RNI=1.0D0-0.5D0*DEXP(ARGTES)
          ENDIF
        ELSE
          ARGTES = D12-(D1-D2*PZOMC)**2
          IF(ARGTES.LT.-2.0D1) THEN
            RNI = 0.D0
          ELSE
            RNI=0.5D0*DEXP(ARGTES)
          ENDIF
        ENDIF
        S0=S0+FZ(M,I)*RNI
      ENDIF
    1 CONTINUE
C
C  ****  SAMPLING TAU.
C
    2 CONTINUE
      IF(rng()*A2.LT.A1) THEN
        TAU=TAU0**rng()
      ELSE
        TAU=DSQRT(1.0D0+rng()*(TAU0*TAU0-1.0D0))
      ENDIF
      CDT1=(1.0D0-TAU)/(EK*TAU)
C  ****  INCOHERENT SCATTERING FUNCTION.
      S=0.0D0
      DO 3 I=1,NSM
      IF(BEN(M,I).LT.energy) THEN
        AUX=energy*(energy-BEN(M,I))*CDT1
        PZOMC=FJ0(M,I)*(AUX-REV*BEN(M,I))
     1       /(REV*DSQRT(AUX+AUX+BEN(M,I)**2))
C
        IF(PZOMC.GT.0.0D0) THEN
          ARGTES = D12-(D1+D2*PZOMC)**2
          IF(ARGTES.LT.-2.0D1) THEN
            RN(I) = 1.0D0
          ELSE
            RN(I)=1.0D0-0.5D0*DEXP(ARGTES)
          ENDIF
        ELSE
          ARGTES = D12-(D1-D2*PZOMC)**2
          IF(ARGTES.LT.-2.0D1) THEN
            RN(I) = 0.D0
          ELSE
            RN(I)=0.5D0*DEXP(ARGTES)
          ENDIF
        ENDIF
        S=S+FZ(M,I)*RN(I)
        PAC(I)=S
      ELSE
        PAC(I)=S-1.0D-6
      ENDIF
    3 CONTINUE
C  ****  REJECTION FUNCTION.
      TST=S*(1.0D0+TAU*(EK1+TAU*(EK2+TAU*EK3)))
     1   /(EK3*TAU*(1.0D0+TAU*TAU))
      IF(rng()*S0.GT.TST) GO TO 2
      costhe=1.0D0-CDT1
C
C  ****  TARGET ELECTRON SHELL.
C
    4 CONTINUE
      TST=S*rng()
      DO 5 I=1,NSM
      IF(PAC(I).GT.TST) THEN
        ISH=I
        GO TO 6
      ENDIF
    5 CONTINUE
      ISH=NSM
    6 CONTINUE
C
C  ****  PROJECTED MOMENTUM OF THE TARGET ELECTRON.
C
      A=rng()*RN(ISH)
      IF(A.LT.0.5D0) THEN
        PZOMC=(D1-DSQRT(D12-DLOG(A+A)))/(D2*FJ0(M,ISH))
      ELSE
        PZOMC=(DSQRT(D12-DLOG(2.0D0-A-A))-D1)/(D2*FJ0(M,ISH))
      ENDIF
      IF(PZOMC.LT.-1.0D0) GO TO 4
C
C  ****  F(EP) REJECTION.
C
      XQC=1.0D0+TAU*(TAU-2.0D0*costhe)
      AF=DSQRT(XQC)*(1.0D0+TAU*(TAU-costhe)/XQC)
      IF(AF.GT.0.0D0) THEN
        FPZMAX=1.0D0+AF*0.2D0
      ELSE
        FPZMAX=1.0D0-AF*0.2D0
      ENDIF
      FPZ=1.0D0+AF*DMAX1(DMIN1(PZOMC,0.2D0),-0.2D0)
      IF(rng()*FPZMAX.GT.FPZ) GO TO 4
C
C  ****  ENERGY OF THE SCATTERED PHOTON.
C
      T=PZOMC**2
      B1=1.0D0-T*TAU*TAU
      B2=1.0D0-T*TAU*costhe
      IF(PZOMC.GT.0.0D0) THEN
        EP=energy*(TAU/B1)*(B2+DSQRT(DABS(B2*B2-B1*(1.0D0-T))))
      ELSE
        EP=energy*(TAU/B1)*(B2-DSQRT(DABS(B2*B2-B1*(1.0D0-T))))
      ENDIF
      efrac = ep/energy
      RETURN
C
C  ****  NO DOPPLER BROADENING FOR E GREATER THAN 5 MeV.
C
    7 CONTINUE
      IF(rng()*A2.LT.A1) THEN
        TAU=TAU0**rng()
      ELSE
        TAU=DSQRT(1.0D0+rng()*(TAU0*TAU0-1.0D0))
      ENDIF
C  ****  REJECTION FUNCTION.
      TST=(1.0D0+TAU*(EK1+TAU*(EK2+TAU*EK3)))
     1   /(EK3*TAU*(1.0D0+TAU*TAU))
      IF(rng().GT.TST) GO TO 7
      EP=TAU*energy
      costhe=1.0D0-(1.0D0-TAU)/(EK*TAU)
C
C  ****  TARGET ELECTRON SHELL.
C
    8 CONTINUE
      TST=ZT(M)*rng()
      S=0.0D0
      DO 9 I=1,NSM
      S=S+FZ(M,I)
      IF(S.GT.TST) THEN
        ISH=I
        GO TO 10
      ENDIF
    9 CONTINUE
      ISH=NSM
   10 CONTINUE
      IF(EP.GT.energy-BEN(M,ISH)) GO TO 7
      efrac = ep/energy
      return
      end
